﻿using System;
using BEPUphysics.Materials;
using BEPUphysics.BroadPhaseEntries;
using BEPUphysics.CollisionShapes;
using BEPUphysics.DataStructures;
using BEPUphysics.Entities.Prefabs;
using BEPUutilities;
using Microsoft.Xna.Framework.Graphics;
using BEPUphysics;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Fires a sphere at a static mesh in a certain way to demonstrate
    /// it passing through the mesh with CCD
    /// </summary>
    public class SphereStaticMeshIssue : StandardDemo
    {
        private float _timer = 0.0f;
        private const float _resetTime = 1.5f;

        private Sphere _activeSphere = null;
 
        private readonly float _friction = 0.5f;
        private Material _groundMaterial = null;
        private Material _sphereMaterial = null;

        private const float _baseballRadius = 0.036322f;
        private const float _baseballMass = 0.148835f;


        public SphereStaticMeshIssue(DemosGame game)
            : base(game)
        {
            BEPUphysics.Settings.CollisionResponseSettings.Softness = 0.0f;


            _groundMaterial = new Material(_friction, _friction, 0.5f);
            _sphereMaterial = new Material(_friction, _friction, 0.9f);


            // Create the ground mesh
            StaticMesh staticMesh = new StaticMesh(GreenTestMesh.Positions, GreenTestMesh.Indices);
            staticMesh.Material = _groundMaterial;
            staticMesh.Sidedness = TriangleSidedness.DoubleSided;


            Space.Add(staticMesh);
            game.ModelDrawer.Add(staticMesh);

            game.Camera.Position = new Vector3(0.0f, 1.0f, 0.0f);
            game.Camera.Yaw(-MathHelper.Pi);

            _timer = _resetTime;
        }


        public override void Update(float dt)
        {
            base.Update(dt);

            _timer += dt;


            if (_timer >= _resetTime)
            {
                // Remove the old sphere before adding a new one
                if(_activeSphere != null)
                {
                    Space.Remove(_activeSphere);
                    Game.ModelDrawer.Remove(_activeSphere);
                    _activeSphere = null;
                }


                // Create a new sphere

                // To demonstrate the ball falling through the ground, the start position and linear velocity
                // can not be changed.
                _activeSphere = new Sphere(new Vector3(0.0f, 1.0f, 5.0f), _baseballRadius, _baseballMass);
                _activeSphere.LinearVelocity = new Vector3(-0.4785156f, 0.6770837f, 10.0f);

                _activeSphere.Material = _sphereMaterial;
                _activeSphere.PositionUpdateMode = BEPUphysics.PositionUpdating.PositionUpdateMode.Continuous;

                // I've seen the issue with non-zero dampings. If you do change these it will change the
                // condition on the first collision and the test case will no longer work
                _activeSphere.LinearDamping = 0.0f;
                _activeSphere.AngularDamping = 0.0f;


                Space.Add(_activeSphere);
                Game.ModelDrawer.Add(_activeSphere);

                _timer = 0.0f;
            }
        }


        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string Name
        {
            get { return "Sphere Static Mesh Issue"; }
        }
    }
}